/*
 *    Copyright 2010 Jonathan Savage
 * 
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml.Serialization;
using System.IO;

namespace ParseTCX
{
    class Program
    {
        static void Main(string[] args)
        {
            if (args.Length == 0)
            {
                Console.Out.WriteLine("Usage: ParseTCX <file.tcx>");
                Console.Out.WriteLine("Press return to contine");
                Console.In.ReadLine();
                return;
            }
            if (args.Length > 1)
            {
                Console.Out.WriteLine("Usage: ParseTCX <file.tcx> (too many parameters - did you forget to quote the filename?)");
                Console.Out.WriteLine("Press return to contine");
                Console.In.ReadLine();
                return;
            }
            TrainingCenterDatabase_t tcx;

            XmlSerializer mySerializer = new XmlSerializer(typeof(TrainingCenterDatabase_t));
            //FileStream myFileStream = new FileStream(@"C:\Documents and Settings\a560507\My Documents\A Temp\12_3_2010 7_02_56 AM_history.tcx", FileMode.Open);
            //FileStream myFileStream = new FileStream(@"C:\Documents and Settings\a560507\My Documents\A Temp\12_1_2010 7_00_32 AM_history.tcx", FileMode.Open);
            if(!File.Exists(args[0]))
            {
                Console.Out.WriteLine("File " + args[0] + " not found");
                Console.Out.WriteLine("Press return to contine");
                Console.In.ReadLine();
                return;
            }
            FileStream myFileStream = new FileStream(args[0], FileMode.Open);
            tcx = (TrainingCenterDatabase_t)mySerializer.Deserialize(myFileStream);
            Decimal sumscale = 0m;
            int numpoints = 0;

            foreach (Activity_t activity in tcx.Activities.Activity)
            {

                foreach (ActivityLap_t lap in activity.Lap)
                {
                    Console.Out.WriteLine("new lap");
                    DateTime lasttime = DateTime.Now;
                    Decimal lastdistance = 0m;
                    bool valid = false;
                    foreach (Trackpoint_t point in lap.Track)
                    {
                        if (point.Extensions != null && 
                            point.Extensions.Any.Length > 0 &&
                            point.Extensions.Any[0].ChildNodes != null &&
                            point.Extensions.Any[0].ChildNodes.Count > 1 &&
                            point.Extensions.Any[0].ChildNodes[0].Name == "Speed" &&
                            point.Extensions.Any[0].ChildNodes[1].Name == "RunCadence")
                        {
                            //got a good one
                            if (valid)
                            {
                                Decimal seconds = (Decimal)point.Time.Subtract(lasttime).TotalSeconds;
                                Decimal currentdistance = (Decimal)point.DistanceMeters;
                                Decimal distance = currentdistance - lastdistance;
                                Decimal gpsSpeed = distance / seconds;
                                Decimal footpodspeedMPS = Decimal.Parse(point.Extensions.Any[0].FirstChild.ChildNodes[0].Value); //Meters per second!
                                if (footpodspeedMPS > 0m && seconds > 0 && distance > 0)
                                {

                                    Decimal footpoddistance = footpodspeedMPS * seconds;
                                    Decimal scale = distance / footpoddistance;
                                    Console.Out.WriteLine("GPS Speed (m/s) " + Decimal.Round(gpsSpeed, 2) + ", Foot Pod Speed (m/s) " + Decimal.Round(footpodspeedMPS, 2) + ", footpod distance " + Decimal.Round(footpoddistance, 2) + ", gps distance " + Decimal.Round(distance, 2) + ", seconds " + seconds + ", scale " + Decimal.Round(scale, 4));
                                    sumscale += scale;
                                    numpoints++;
                                }
                            }
                            lasttime = point.Time;
                            lastdistance = (Decimal)point.DistanceMeters;
                            valid = true;
                        }
                    }
                }
            }
            Console.Out.WriteLine("average scale " + Decimal.Round(sumscale / numpoints, 8));

            Console.Out.WriteLine("Press return to contine");
            Console.In.ReadLine();
        }
    }
}
